﻿<# 	
	************************************************************************************************************
	Copyright (c) Microsoft Corporation.  All rights reserved.
	
	Version: 	1.0
	
	Purpose:	Configure Deployment Share of Microsoft Deploymnet Toolkit
	************************************************************************************************************
#>

param
(
    [Parameter(Mandatory=$true)][String] $DSTargetPath,
    [Parameter(Mandatory=$true)][String] $DSShareName,
    [Parameter(Mandatory=$false)][String] $AddBootWimToWDS,
    [Parameter(Mandatory=$false)][String] $WindowsSource,
    [ValidateSet("YES","NO")][Parameter(Mandatory=$false)][String] $SurfaceFirmware="NO",
    [ValidateSet("YES","NO")][Parameter(Mandatory=$false)][String] $SurfaceApp="NO",
    [ValidateSet("YES","NO")][Parameter(Mandatory=$false)][String] $SurfacePro3Drivers="No"

)

$DSShareDescription = "Microsoft Surface Deployment Accelerator"
Write-Host "Surface Pro 3: $SurfacePro3Drivers"

# Get the current path of the script
Function Get-ScriptPath
{
    $Invocation = (Get-Variable MyInvocation -Scope 1).Value
    Split-Path $Invocation.MyCommand.Path
}

# Create the folder and SMB share for the new deployment share
Function New-SurfaceDeploymentShare
{
    If (!(Test-Path -Path $DSTargetPath))
    {
        Write-Host "Creating new deployment share folder"
        New-Item -Path $DSTargetPath -ItemType Directory | Out-Null
        Try
        {
        
            if ((((Get-SmbShare | ? {$_.Name -eq $DSShareName}) | Measure-Object).count) -eq 0)
            {
                Write-Host "Creating SMB Share for deployment share located at $DSTargetPath."
                New-SmbShare -Name $DSShareName -Path $DSTargetPath -Description $DSShareDescription -FullAccess "Everyone" | Out-Null
            }
            else
            {
                Write-Host "Share name is already in use."
                Exit -1
            }
        }
        catch
        {
            Write-Host "Unable to create SMB share for $DSTargetPath. Exiting script."
            Exit -1
        }      
    }
    Else
    {
        
        if ((((Get-ChildItem $DSTargetPath -Recurse) | Measure-Object).Count) -eq 0)
        {
            Write-Host "Using user-created folder at $DSTargetPath."
            Try
            {
        
                if ((((Get-SmbShare | ? {$_.Name -eq $DSShareName}) | Measure-Object).count) -eq 0)
                {
                    Write-Host "Creating SMB Share for deployment share located at $DSTargetPath."
                    New-SmbShare -Name $DSShareName -Path $DSTargetPath -Description $DSShareDescription -FullAccess "Everyone" | Out-Null
                }
                else
                {
                    Write-Host "Share name is already in use."
                    Exit -1
                }
            }
            catch
            {
                Write-Host "Unable to create SMB share for $DSTargetPath. Exiting script."
                Exit -1
            }      
        }
        else
        {
            Write-Host "Found existing deploy share at $DSTargetPath. Unable to progress."
            Exit -1
        }
    }
      
}
#Update boot.ini with user-provided parameters and deployroot
Function Update-BootIni
{
    $BootIni = "$DSTargetPath\Control\Bootstrap.ini"   
    Write-Host "Updating boot.ini at $BootIni" 
    Add-Content -Path $BootIni -Value "DeployRoot=\\$HostName\$DSShareName"
    Add-Content -Path $BootIni -Value "UserID=$DSUserName"
    Add-Content -Path $BootIni -Value "UserDomain=$DSUserDomain"
    Add-Content -Path $BootIni -Value "UserPassword=$DSUserPassword"      
}

#Download the file at the specified URl to the specified location locally
function Download-File
{  
    param(
        [Parameter(Mandatory=$true)]
        [String] $url,
        [Parameter(Mandatory=$false)]
        [String] $localFile = (Join-Path $pwd.Path $url.SubString($url.LastIndexOf('/'))) 
    )
    
    begin {
	    $client = New-Object System.Net.WebClient
        $Global:downloadComplete = $false

        $eventDataComplete = Register-ObjectEvent $client DownloadFileCompleted `
            -SourceIdentifier WebClient.DownloadFileComplete `
            -Action {$Global:downloadComplete = $true}

        $eventDataProgress = Register-ObjectEvent $client DownloadProgressChanged `
            -SourceIdentifier WebClient.DownloadProgressChanged `
            -Action { $Global:DPCEventArgs = $EventArgs }    
    }

    process {
        Write-Progress -Activity 'Downloading file' -Status $url
        $client.DownloadFileAsync($url, $localFile)
    
        while (!($Global:downloadComplete)) {                
            $pc = $Global:DPCEventArgs.ProgressPercentage
            if ($pc -ne $null) {
                Write-Progress -Activity 'Downloading file' -Status $url -PercentComplete $pc
            }
        }
    
        Write-Progress -Activity 'Downloading file' -Status $url -Complete
    }

    end {
        Unregister-Event -SourceIdentifier WebClient.DownloadProgressChanged
        Unregister-Event -SourceIdentifier WebClient.DownloadFileComplete
        $client.Dispose()
        $Global:downloadComplete = $null
        $Global:DPCEventArgs = $null
        Remove-Variable client
        Remove-Variable eventDataComplete
        Remove-Variable eventDataProgress
        [GC]::Collect() 
    }
}  

# Check if there is an internet connection
Function Check-Internet
{
    While (([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]‘{DCB00C01-570F-4A9B-8D69-199FDBA5723B}’)).IsConnectedToInternet) -eq $False)
    {
        Write-Host "No internet connection detected. Retrying in 60 seconds..."
        Start-Sleep -Seconds 60
    }
}

# Extract the content of a compressed archive to the specified folder
Function Extract-Zip
{
    param(
        [Parameter(Mandatory=$true)][String] $ZIPFilePath,
        [Parameter(Mandatory=$true)][String] $ExtractionPath 
    )    
    begin 
    {
	    $Shell = new-object -com shell.application
        $ZipFile = $Shell.NameSpace($ZIPFilePath)  
    }
    process 
    {
        If (!(Test-Path -Path $ExtractionPath))
        {
            New-Item -Path $ExtractionPath -ItemType Directory | Out-Null
        }
        foreach($Item in $ZipFile.items())
        {
            $Shell.Namespace($ExtractionPath).copyhere($Item,0x14) | Out-Null
        }    
    }
}

# Interim function to control the download and extraction of files for the device
Function Get-SurfaceFile
{
    param(
        [Parameter(Mandatory=$true)][String] $URL,
        [Parameter(Mandatory=$true)][String] $FileName, 
        [Parameter(Mandatory=$true)][String] $LocalFolder,
        [Parameter(Mandatory=$true)][String] $DownloadTitle
    )

    $LocalFile = Join-Path -Path $LocalFolder -ChildPath $FileName    
    If (Test-Path -Path $LocalFile)
    {
        #Write-Host "Found $FileName at $LocalFolder"
    }
    else
    {
        Write-Host "Downloading $DownloadTitle"
        Download-File $url $LocalFile
    }
}

# Intermin function for extracting content from a compressed archive file
Function Extract-Package
{
    param(
        [Parameter(Mandatory=$true)][String] $ZipFileName, 
        [Parameter(Mandatory=$true)][String] $ExtractPath,
        [Parameter(Mandatory=$true)][String] $ExtractTitle
        )
    Write-Host "Extracting $ExtractTitle"
    Extract-Zip -ZIPFilePath $ZipFileName -ExtractionPath $ExtractPath
}

# Import OS Packages into MDT
Function Import-OSPackage
{
    param(
        [Parameter(Mandatory=$true)]
        [String]$OSPackagePath
    )
    $DSPkgTarget = "$MDTPSDriveName"+':\Packages\OS Patches\x64 Windows 8.1'
    Import-MDTPackage -Path $DSPkgTarget -SourcePath $OSPackagePath | Out-Null
}
 
# Download all user-selected Microsoft Surface files. Requires internet connection
Function Get-SurfaceDrivers
{
    Check-Internet

    If ($SurfaceFirmware -eq "YES")
    {
        $TargetFolder = "$DSTargetPath\Applications\INSTALL - Surface Firmware Tool\Source"
        If (-Not (Test-Path -Path (Join-Path -Path $TargetFolder -ChildPath "SurfaceFirmwareTool.msi")))
        {
            Get-SurfaceFile -URL "http://aka.ms/SurfaceFirmwareDownload" -FileName "SurfaceFirmwareTool.msi" -LocalFolder $TargetFolder -DownloadTitle "Surface Firmware Tool"
        }
    }
    If ($SurfaceApp -eq "YES")
    {
        $TargetFolder = "$DSTargetPath\Applications\INSTALL - Surface Application"
        If (-Not (Test-Path -Path (Join-Path -Path $TargetFolder -ChildPath "SurfaceApp.zip")))
        {
            Get-SurfaceFile -URL "http://aka.ms/surfaceappdownload" -FileName "SurfaceApp.zip" -LocalFolder $TargetFolder -DownloadTitle "Surface Application" 
            Extract-Package -ZipFileName (Join-Path -Path $TargetFolder -ChildPath "SurfaceApp.zip") -ExtractPath (Join-Path -Path $TargetFolder -ChildPath "Source") -ExtractTitle "Surface Application" 
        }
    }
    If ($SurfacePro3Drivers -eq "YES")
    {
        $TargetFolder = "$DSTargetPath\Applications\CONFIG - Surface Drivers"
        If (-Not (Test-Path -Path (Join-Path -Path $TargetFolder -ChildPath "SurfacePro3DriverPack.zip")))
        {
            Get-SurfaceFile -URL "http://aka.ms/SurfacePro3DriverDownload" -FileName "SurfacePro3DriverPack.zip" -LocalFolder $TargetFolder -DownloadTitle "Surface Pro 3 Driver Pack" 
            Extract-Package -ZipFileName (Join-Path -Path $TargetFolder -ChildPath "SurfacePro3DriverPack.zip") -ExtractPath (Join-Path -Path $TargetFolder -ChildPath "Source\Surface Pro 3") -ExtractTitle "Surface Pro 3 Driver Pack"
        }
        $TargetFolder = "$DSTargetPath\Applications\CONFIG - Surface Pro 3 Asset Tag"
        If (-Not (Test-Path -Path (Join-Path -Path $TargetFolder -ChildPath "SurfacePro3AssetTagTool.zip")))
        {
            Get-SurfaceFile -URL "http://aka.ms/SurfacePro3AssetTagDownload" -FileName "SurfacePro3AssetTagTool.zip" -LocalFolder $TargetFolder -DownloadTitle "Surface Pro 3 Asset Tag Tool" 
            Extract-Package -ZipFileName (Join-Path -Path $TargetFolder -ChildPath "SurfacePro3AssetTagTool.zip") -ExtractPath (Join-Path -Path $TargetFolder -ChildPath "Source") -ExtractTitle "Surface Pro 3 Asset Tag Tool"
        }    
        $TargetFolder = "$DSTargetPath\Applications\CONFIG - Surface Drivers\Source\Surface Pro 3\OSPackage"
        If (-Not (Test-Path -Path (Join-Path -Path $TargetFolder -ChildPath "Windows8.1-KB296817x64_ClickNoteFix.msup")))
        {
            If (-Not (Test-Path -Path $TargetFolder))
            {
                New-Item -Path $TargetFolder -ItemType Directory | Out-Null
            }
            Get-SurfaceFile -URL "http://aka.ms/surfacepro3clicknotefixdownload" -FileName "Windows8.1-KB296817x64_ClickNoteFix.msu" -LocalFolder $TargetFolder -DownloadTitle "Surface Pro 3 Click Note Fix"     
            Import-OSPackage -OSPackagePath $TargetFolder
        }
    }
}

# Import Windows PE drivers into MDT
Function Import-WinPEDrivers
{    
    $SurfacePro3_Ethernet="EthernetAdapter"
    $TargetFolder = "$DSTargetPath\Applications\CONFIG - Surface Drivers\Source"
    $DSImportPath = "$MDTPSDriveName"+':\Out-of-Box Drivers\WinPE x64'
    $NetworkAdapterDriverfolders = Get-ChildItem -Path $TargetFolder -Include "*$SurfacePro3_Ethernet*" -Recurse 
    ForEach ($Folder in $NetworkAdapterDriverfolders)
    {
        Import-MDTDriver -Path $DSImportPath -SourcePath $Folder | Out-Null
    }
}

# Import a boot wim into the local WDS server
Function Import-WDSBootWim
{
   $BootWim = Join-Path -Path $DSTargetPath -ChildPath "Boot\LiteTouchPE_x64.wim"
   $WDSBootDescription = "Microsoft Surface Deployment Accelerator"
   $WDSFileName = "SurfaceDeploymentAccelerator.wim"
   Write-Host "Adding boot image to WDS Server"
   wdsutil /add-image /imagefile:$BootWim /imagetype:boot /Name:"$WDSBootDescription"  /Filename:"$WDSFileName"  | out-null

}

# Import non-boot critical drivers into MDT
Function Import-ModelDrivers
{
    
    
    If ($SurfacePro3Drivers -eq "YES") 
    {
        Write-Host "Device model is Surface Pro 3"
        $DeviceModel = "Surface Pro 3"
    }
    Else {Write-Host "Unknown device. Unable to proceed."; Exit -1}
    $DSImportPath = "$MDTPSDriveName"+':\Out-of-Box Drivers\'+$DeviceModel
    $TargetFolder = "$DSTargetPath\Applications\CONFIG - Surface Drivers\Source\$DeviceModel"
    Import-MDTDriver -Path $DSImportPath -SourcePath $TargetFolder | Out-Null
}

# Check the version of MDT is suitable
Function Get-MDTVersion
{
    $MDT = $InstalledProducts | Where-Object {($_.Name -like "*Microsoft Deployment*") -and ($_.Version -eq "6.2.5019.0")}  
    If ($MDT -eq $Null)
    {
        Write-Host "MDT is either not installed or is the incorrect version."
        Return $false
        
    }
    else
    {
        Write-Host "Installed MDT version is appropriate."
        Return $true
    }
}

# Check the version of the Windows ADK is suitable
Function Get-WindowsADKVersion
{
    $WindowsDeploymentTools = $InstalledProducts | Where-Object {($_.Name -eq "Windows Deployment Tools") -and ($_.Version -like "*8.100*")} 
    If ($WindowsDeploymentTools -eq $Null)
    {
        Write-Host "Windows ADK is either not installed or not the correct version."
        Return $False
    }
    else
    {
        Write-Host "Windows ADK is installed and is the correct version."
        Return $True
    }          
}

# Check the user-provided Windows ource file path is accessible
Function Check-OSSourcePath
{
    $TestPath = Join-Path -Path $WindowsSource -ChildPath "sources\install.wim"
    If (Test-Path -Path $TestPath)
    {
        Write-Host "Windows source file location exists."
        Return $true
    }
    else
    {
        Write-Host "Unable to locate the Windows source files at $WindowsSource."
        Return $false
    }
}


#Update the OS GUID in the TS
Function Update-OSGuid
{
    # Load the OperatingSystems.xml file to get the GUID of the newly imported OS
    [xml]$OSfile = Get-Content "$DSTargetPath\Control\OperatingSystems.xml"
    foreach($os in $OSfile.oss.os) 
        {
        $OSGUID = $os.guid 
        }
    # If the GUID is blank (for example if the OperatingSystem.xml file didn't get formed or import was cancelled), add a template GUID
        If ($OSGUID -eq $Null)
        {
        $OSGUID = "{11111111-aaaa-bbb-cccc-222222222222}"
        }
    # Load the TS.xml file that has been copied to the ICS distribution share
    $doc = New-Object System.Xml.XmlDocument
    $doc.Load("$DSTargetPath\Control\x64_Surface\ts.xml")
    # Re-write the OSGUID attributes with the GUID retrieved from OperatingSystems.xml
    $doc.SelectSingleNode("//sequence/globalVarList/variable[@name='OSGUID']").InnerText = $OSGUID
    $doc.SelectSingleNode("//sequence/group[@name='Install']/step[@type='BDD_InstallOS']/defaultVarList/variable[@name='OSGUID']").InnerText = $OSGUID
    $doc.Save("$DSTargetPath\Control\x64_Surface\ts.xml")
    write-host "The GUID for the imported original OS source has been added to the Image Creation Service (ICS) Task Sequence" -ForegroundColor green
}

# Control routine for all pre-flight verification
Function Verify-Environment
{
    Write-Host "Verifying system..."
    If ((Get-MDTVersion) -and (Get-WindowsADKVersion) -and (Check-OSSourcePath))
    {
        Write-Host "Environment verified. Configuration will proceed."
    }
    else
    {
        Write-Host "One or more pre-requisites could not be verified. Please remediate and re-run the tool."
        Exit -1
    }
}

# Control Routine
$HostName = $env:COMPUTERNAME
$MDTPSDriveName = "SurfaceAccelerator"
$MDTPSDrive = "$MDTPSDriveName"+':\'
$DSSourcePath = Join-Path -Path (Get-ScriptPath) -ChildPath "DSTemplate"
$InstalledProducts = Get-WmiObject Win32_Product        
Verify-Environment
Write-Host "Starting configuration of the Surface Deployment Accelerator..."
Add-PSSnapIn Microsoft.BDD.PSSnapIn      
# Setup deployment share    
New-SurfaceDeploymentShare
Write-Host "Populating target with template content"
robocopy $DSSourcePath $DSTargetPath /mir | out-null 
Write-Host "Creating new MDT deployment share for Surface devices."
New-PSDrive -Name  $MDTPSDriveName -PSProvider "MDTProvider" -Root "$DSTargetPath" -Description "Microsoft Surface Deployment Accelerator" -NetworkPath "\\$HostName\$DSShareName" | add-MDTPersistentDrive | Out-Null
Write-Host "Performing deployment share configuration."
Set-ItemProperty $MDTPSDrive -Name PhysicalPath -Value $DSTargetPath | out-null
Set-ItemProperty $MDTPSDrive -name Boot.x86.GenerateLiteTouchISO -Value "False" | out-null
$DSOSTarget = "$MDTPSDriveName"+':\Operating Systems\Windows 8.1'    
If (($WindowsSource -ne $null) -and (Test-Path -Path $WindowsSource))
{
    Write-Host "Importing operating system from $WindowsSource"
    Import-MDTOperatingSystem -Path $DSOSTarget -SourcePath $WindowsSource -DestinationFolder "x64 Windows 8.1 Enterprise With Update"  | Out-Null
}    
# Update bootstrap.ini in the new deployment share
Update-BootIni
# Download and setup drivers and apps for the Surface device that was selected
Get-SurfaceDrivers
# Import the Windows PE drivers into the new deployment share
Write-Host "Importing boot-critical drivers for Windows PE."
Import-WinPEDrivers
Start-Sleep -Seconds 30
Write-Host "Importing model-specific drivers."    
Import-ModelDrivers   
Start-Sleep -Seconds 30
# Update the new deployment share
Write-Host "Creating Windows PE boot image."
Update-MDTDeploymentShare -path $MDTPSDrive -Force | Out-Null
# If requested, import the boot.wim that was created into WDS
If ($AddBootWimToWDS)
{
    Import-WDSBootWim
}
Write-Host "Updating OS GUID for the task sequence."
Update-OSGuid 
If ($SurfacePro3Drivers) {$Devicename = "Surface Pro 3"}
else {}
    
Write-Host "Microsoft Surface Deployment Accelerator is now ready for use. Please PXE boot a connected $Devicename device."