<# 	
	************************************************************************************************************
	Copyright (c) Microsoft Corporation.  All rights reserved.
	
	Microsoft Consulting Services - JumpStart Program
	
	File:		INSTALL_ADK.ps1
	
	Version: 	1.0
	
	Purpose:	Installs the Windows Automated Deployment Toolkit
	
	Pre-Reqs:	Windows Server 2012
				Windows PowerShell 3.0
	
	************************************************************************************************************
#>
function Download-File
{  
    param(
        [Parameter(Mandatory=$true)]
        [String] $url,
        [Parameter(Mandatory=$false)]
        [String] $localFile = (Join-Path $pwd.Path $url.SubString($url.LastIndexOf('/'))) 
    )
    
    begin {
	    $client = New-Object System.Net.WebClient
        $Global:downloadComplete = $false

        $eventDataComplete = Register-ObjectEvent $client DownloadFileCompleted `
            -SourceIdentifier WebClient.DownloadFileComplete `
            -Action {$Global:downloadComplete = $true}

        $eventDataProgress = Register-ObjectEvent $client DownloadProgressChanged `
            -SourceIdentifier WebClient.DownloadProgressChanged `
            -Action { $Global:DPCEventArgs = $EventArgs }    
    }

    process {
        Write-Progress -Activity 'Downloading file' -Status $url
        $client.DownloadFileAsync($url, $localFile)
    
        while (!($Global:downloadComplete)) {                
            $pc = $Global:DPCEventArgs.ProgressPercentage
            if ($pc -ne $null) {
                Write-Progress -Activity 'Downloading file' -Status $url -PercentComplete $pc
            }
        }
    
        Write-Progress -Activity 'Downloading file' -Status $url -Complete
    }

    end {
        Unregister-Event -SourceIdentifier WebClient.DownloadProgressChanged
        Unregister-Event -SourceIdentifier WebClient.DownloadFileComplete
        $client.Dispose()
        $Global:downloadComplete = $null
        $Global:DPCEventArgs = $null
        Remove-Variable client
        Remove-Variable eventDataComplete
        Remove-Variable eventDataProgress
        [GC]::Collect() 
    }
}  

Function Check-Internet
{
    While (([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]{DCB00C01-570F-4A9B-8D69-199FDBA5723B})).IsConnectedToInternet) -eq $False)
    {
        Write-Host "No internet connection detected. Retrying in 60 seconds..."
        Start-Sleep -Seconds 60
    }
}


# *****************************************************************************
# MAIN()
# *****************************************************************************
# Common Processing
# *****************************************************************************

# Install ADK
$SourceFile = ".\source\adksetup.exe"
$ADKArguments = "  /features OptionID.DeploymentTools OptionID.ApplicationCompatibilityToolkit OptionID.WindowsPreinstallationEnvironment OptionID.UserstateMigrationTool /quiet"
$InstalledProducts = Get-WmiObject Win32_Product
$WindowsDeploymentTools = $InstalledProducts | Where-Object {($_.Name -eq "Windows Deployment Tools") -and ($_.Version -like "*8.100*")}
If ($WindowsDeploymentTools -eq $null)
{
    If ((Test-Path -Path $SourceFile) -eq $true)
    { 
	    Write-Host "Found  Installation files for ADK"
	    $SourceFilePath = $(Get-Item $SourceFile).FullName
    }	
    Else 
    {
	    Write-Host "Downloading Windows ADK"
        Check-Internet
        $url = 'http://download.microsoft.com/download/6/A/E/6AEA92B0-A412-4622-983E-5B305D2EBE56/adk/adksetup.exe'
        $local = "$env:TEMP\adksetup.exe"

        #Download the ADK
        Download-File $url $local

        $SourceFilePath = $(Get-Item $local).FullName
    }

    try {
        Write-Host "Installing Windows Assessment and Deployment Kit"
	    Start-Process -File  $SourceFilePath -Arg $ADKArguments -passthru | wait-process

	    Write-Host  "$AppName - INSTALLATION SUCCESSFULLY COMPLETED" 

	    }
	    catch {

	    Write-Host  "$AppName - INSTALLATION ERROR (ERRORLEVEL=%ERRORLEVEL%)"

    }
}
Else
{
    Write-Host "Windows ADK is already installed"
}

Exit